@@ -75,12 +75,13 @@ class ChEMBLTargetCofactorProvider(StashableBase):
         logger.info("Completed reload with status (%r) at %s (%.4f seconds)", ok, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), time.time() - startTime)
         return fD
 
-    def buildCofactorList(self, sequenceMatchFilePath, crmpObj=None, maxActivity=5):
+    def buildCofactorList(self, sequenceMatchFilePath, crmpObj=None, lnmpObj=None, maxActivity=5):
         """Build target cofactor list for the matching entities in the input sequence match file.
 
         Args:
             sequenceMatchFilePath (str): sequence match output file path
             crmpObj (obj, optional): instance of ChemRefMappingProviderObj()
+            lnmpObj (obj, optional): instance of LigandNeighborMappingProviderObj(). Defaults to None.
             maxActivity (int, optional): maximum number of prioritized activity records per target
 
         Returns:
@@ -114,38 +115,40 @@ class ChEMBLTargetCofactorProvider(StashableBase):
         mD = self.__mU.doImport(sequenceMatchFilePath, fmt="json")
         #
         chP = ChEMBLTargetProvider(cachePath=self.__cachePath, useCache=False)
-        # --- cofactor list
-        chemblIdList = []
-        for queryId, matchDL in mD.items():
-            tS = queryId.split("|")[2]
-            tL = tS.split(",")
-            chemblIdList.extend(tL)
-        chemblIdList = list(set(chemblIdList))
-        logger.info("Total cofactors for matched targets (%d)", len(chemblIdList))
         # ---
-        #
         chaP = ChEMBLTargetActivityProvider(cachePath=self.__cachePath, useCache=True)
         #
         provenanceSource = "ChEMBL"
         refScheme = "PDB entity"
         assignVersion = chP.getAssignmentVersion()
         for queryId, matchDL in mD.items():
-            unpId = queryId.split("|")[0].strip()
-            queryTaxId = queryId.split("|")[4]
-            chemblIdL = queryId.split("|")[2].split(",")
+            qCmtD = self.__decodeComment(queryId)
+            unpId = qCmtD["uniprotId"]
+            queryTaxId = qCmtD["taxId"] if "taxId" in qCmtD else None
+            chemblIdL = qCmtD["chemblId"].split(",")
             if queryTaxId == "-1":
-                logger.info("Skipping target %r (%r)", unpId, chemblIdL)
+                logger.info("Skipping target with missing taxonomy %r (%r)", unpId, chemblIdL)
                 continue
             queryName = chP.getTargetDescription(unpId)
             for chemblId in chemblIdL:
                 if not chaP.hasTargetActivity(chemblId):
                     logger.debug("Skipping target %r (%r)", unpId, chemblId)
-                    continue
-                #
+                    # continue
+                # --
+                chemCompNeighborsD = {}
+                if lnmpObj:
+                    for matchD in matchDL:
+                        tCmtD = self.__decodeComment(matchD["target"])
+                        entryId = tCmtD["entityId"].split("_")[0]
+                        entityId = tCmtD["entityId"].split("_")[1]
+                        rcsbEntityId = entryId + "_" + entityId
+                        chemCompIdList = lnmpObj.getLigandNeighbors(rcsbEntityId)
+                        chemCompNeighborsD.update({k: True for k in chemCompIdList})
+                # --
                 for matchD in matchDL:
-                    tL = matchD["target"].split("|")
-                    entryId = tL[0].split("_")[0]
-                    entityId = tL[0].split("_")[1]
+                    tCmtD = self.__decodeComment(matchD["target"])
+                    entryId = tCmtD["entityId"].split("_")[0]
+                    entityId = tCmtD["entityId"].split("_")[1]
                     #
                     taDL = chaP.getTargetActivity(chemblId)
                     logger.debug("Target %r has (%d) activity records", chemblId, len(taDL))
@@ -169,13 +172,15 @@ class ChEMBLTargetCofactorProvider(StashableBase):
                                         "moa": taD["moa"],
                                         "max_phase": taD["max_phase"],
                                     }
-
+                                    actD = self.__addLocalIds(actD, crmpObj=crmpObj)
                                     actL.append(actD)
                                 except Exception as e:
                                     logger.exception("Failing for tAD %r with %s", taD, str(e))
 
                     # ---
-                    actL = self.__activityListSelect(actL, crmpObj, maxActivity=maxActivity)
+                    actL = self.__activityListSelect(actL, chemCompNeighborsD, maxActivity=maxActivity)
+                    if not actL:
+                        logger.debug("No ChEMBL cofactors for %s %s", chemblId, unpId)
                     # ---
                     rD = {
                         "entry_id": entryId,
@@ -187,8 +192,8 @@ class ChEMBLTargetCofactorProvider(StashableBase):
                         "provenance_source": provenanceSource,
                         "reference_scheme": refScheme,
                         "assignment_version": assignVersion,
-                        "query_taxonomy_id": int(queryTaxId),
-                        "target_taxonomy_id": int(matchD["targetTaxId"]),
+                        "query_taxonomy_id": int(queryTaxId) if queryTaxId else None,
+                        "target_taxonomy_id": int(matchD["targetTaxId"]) if "targetTaxId" in matchD else None,
                         "target_beg_seq_id": matchD["targetStart"],
                         "query_beg_seq_id": matchD["queryStart"],
                         "align_length": matchD["alignLen"],
@@ -212,30 +217,50 @@ class ChEMBLTargetCofactorProvider(StashableBase):
         ok = self.__mU.doExport(fp, {"version": vS, "created": tS, "cofactors": qD}, fmt="json", indent=3)
         return ok
 
-    def __activityListSelect(self, activityDL, crmpOb=None, maxActivity=5):
+    def __addLocalIds(self, cfD, crmpObj=None):
+        #
+        if crmpObj:
+            localIdL = crmpObj.getLocalIds("CHEMBL", cfD["cofactor_id"])
+            if localIdL:
+                localId = localIdL[0]
+                if localId.startswith("PRD_"):
+                    cfD["prd_id"] = localId
+                else:
+                    cfD["chem_comp_id"] = localId
+        return cfD
+
+    def __activityListSelect(self, activityDL, chemCompNeighborsD, maxActivity=5):
         retL = []
-        mappedL = []
+        mappedNeighborL = []
         unmappedL = activityDL
-        if crmpOb:
+        #
+        if chemCompNeighborsD:
             unmappedL = []
-            # Select out the any cases for molecules that map to a local CC or BIRD
+            # Select out the any cases for molecules that map to a neighbor chemical component.
             for activityD in activityDL:
-                localIdL = crmpOb.getLocalIds("CHEMBL", activityD["cofactor_id"])
-                if localIdL:
-                    # add the local identifiers
-                    for localId in localIdL:
-                        if localId.startswith("PRD_"):
-                            activityD.setdefault("prd_id", []).append(localId)
-                        else:
-                            activityD.setdefault("chem_comp_id", []).append(localId)
-                    mappedL.append(activityD)
+                if "chem_comp_id" in activityD and activityD["chem_comp_id"] in chemCompNeighborsD:
+                    activityD["neighbor_in_pdb"] = "Y"
+                    mappedNeighborL.append(activityD)
                 else:
                     unmappedL.append(activityD)
+                    activityD["neighbor_in_pdb"] = "N"
         #
-        numLeft = maxActivity - len(mappedL)
+        numLeft = maxActivity - len(mappedNeighborL)
         if numLeft > 0:
             unmappedL = sorted(unmappedL, key=lambda k: k["measurement_value"], reverse=True)
-            retL = mappedL
+            retL = mappedNeighborL
             retL.extend(unmappedL[:numLeft])
             retL = sorted(retL, key=lambda k: k["measurement_value"], reverse=True)
+        else:
+            logger.debug("Mapped neighbor cofactors (%d) excluded unmapped (%d)", len(mappedNeighborL), len(unmappedL))
+            retL = sorted(mappedNeighborL, key=lambda k: k["measurement_value"], reverse=True)
         return retL
+
+    def __decodeComment(self, comment, separator="|"):
+        dD = {}
+        try:
+            ti = iter(comment.split(separator))
+            dD = {tup[1]: tup[0] for tup in zip(ti, ti)}
+        except Exception:
+            pass
+        return dD
