@@ -58,7 +58,7 @@ class DrugBankTargetCofactorProvider(StashableBase):
         fExt = "json" if fmt == "json" else "pic"
         return os.path.join(self.__dirPath, "drugbank-cofactor-data.%s" % fExt)
 
-    def __reload(self, dirPath, useCache, fmt, **kwargs):
+    def __reload(self, dirPath, useCache, fmt):
         startTime = time.time()
         fD = {}
 
@@ -75,12 +75,13 @@ class DrugBankTargetCofactorProvider(StashableBase):
         logger.info("Completed reload (%r) at %s (%.4f seconds)", ok, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), time.time() - startTime)
         return fD
 
-    def buildCofactorList(self, sequenceMatchFilePath, crmpObj=None):
+    def buildCofactorList(self, sequenceMatchFilePath, crmpObj=None, lnmpObj=None):
         """Build target cofactor list for the matching entities in the input sequence match file.
 
         Args:
             sequenceMatchFilePath (str): sequence match output file path
-            crmpObj (obj, optional): instance of ChemRefMappingProviderObj()
+            crmpObj (obj, optional): instance of ChemRefMappingProviderObj(). Defaults to None
+            lnmpObj (obj, optional): instance of LigandNeighborMappingProviderObj(). Defaults to None.
 
         Returns:
             bool: True for success or False otherwise
@@ -93,15 +94,29 @@ class DrugBankTargetCofactorProvider(StashableBase):
         refScheme = "PDB entity"
         assignVersion = str(dbP.getAssignmentVersion())
         for queryId, matchDL in mD.items():
-            unpId = queryId.split("|")[0]
-            queryTaxId = queryId.split("|")[2].strip()
+            qCmtD = self.__decodeComment(queryId)
+            unpId = qCmtD["uniprotId"]
+            queryTaxId = qCmtD["taxId"] if "taxId" in qCmtD else None
             if not dbP.hasCofactor(unpId) or queryTaxId == "-1":
                 logger.info("Skipping target %r", unpId)
                 continue
+            #
+            # --
+            chemCompNeighborsD = {}
+            if lnmpObj:
+                for matchD in matchDL:
+                    tCmtD = self.__decodeComment(matchD["target"])
+                    entryId = tCmtD["entityId"].split("_")[0]
+                    entityId = tCmtD["entityId"].split("_")[1]
+                    rcsbEntityId = entryId + "_" + entityId
+                    chemCompIdList = lnmpObj.getLigandNeighbors(rcsbEntityId)
+                    chemCompNeighborsD.update({k: True for k in chemCompIdList})
+            # --
+            #
             for matchD in matchDL:
-                tL = matchD["target"].split("|")
-                entryId = tL[0].split("_")[0]
-                entityId = tL[0].split("_")[1]
+                tCmtD = self.__decodeComment(matchD["target"])
+                entryId = tCmtD["entityId"].split("_")[0]
+                entityId = tCmtD["entityId"].split("_")[1]
                 # --
                 dbDL = dbP.getCofactors(unpId)
                 # --
@@ -117,7 +132,14 @@ class DrugBankTargetCofactorProvider(StashableBase):
                     cfD["inchi_key"] = dbD["inchi_key"]
                     cfD["smiles"] = dbD["smiles"]
                     cfD["pubmed_ids"] = dbD["pubmed_ids"]
-                    cfDL.append(self.__addLocalIds(cfD, crmpObj))
+                    cfD = self.__addLocalIds(cfD, crmpObj)
+                    #
+                    if "chem_comp_id" in cfD and cfD["chem_comp_id"] in chemCompNeighborsD:
+                        cfD["neighbor_in_pdb"] = "Y"
+                    else:
+                        cfD["neighbor_in_pdb"] = "N"
+                    #
+                    cfDL.append(cfD)
                 # --
                 query_name = cfDL[0]["target_name"] if cfDL and "target_name" in cfDL[0] else None
                 # --
@@ -131,8 +153,8 @@ class DrugBankTargetCofactorProvider(StashableBase):
                     "provenance_source": provenanceSource,
                     "reference_scheme": refScheme,
                     "assignment_version": assignVersion,
-                    "query_taxonomy_id": int(queryTaxId),
-                    "target_taxonomy_id": int(matchD["targetTaxId"]),
+                    "query_taxonomy_id": int(queryTaxId) if queryTaxId else None,
+                    "target_taxonomy_id": int(matchD["targetTaxId"]) if "targetTaxId" in matchD else None,
                     "target_beg_seq_id": matchD["targetStart"],
                     "query_beg_seq_id": matchD["queryStart"],
                     "align_length": matchD["alignLen"],
@@ -166,3 +188,12 @@ class DrugBankTargetCofactorProvider(StashableBase):
                 else:
                     cfD["chem_comp_id"] = localId
         return cfD
+
+    def __decodeComment(self, comment, separator="|"):
+        dD = {}
+        try:
+            ti = iter(comment.split(separator))
+            dD = {tup[1]: tup[0] for tup in zip(ti, ti)}
+        except Exception:
+            pass
+        return dD
