@@ -37,7 +37,7 @@ class PharosTargetCofactorProvider(StashableBase):
         #
 
     def testCache(self, minCount=1):
-        logger.info("Pharos cofactor count %d", len(self.__fD["cofactors"]) if "cofactors" in self.__fD else 0)
+        logger.info("Pharos cached cofactor count %d", len(self.__fD["cofactors"]) if "cofactors" in self.__fD else 0)
         if self.__fD and "cofactors" in self.__fD and len(self.__fD["cofactors"]) > minCount:
             return True
         else:
@@ -70,16 +70,20 @@ class PharosTargetCofactorProvider(StashableBase):
             fU = FileUtil()
             fU.mkdir(dirPath)
         # ---
-        logger.info("Completed reload with status (%r) at %s (%.4f seconds)", ok, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), time.time() - startTime)
+        numCofactors = len(fD["cofactors"]) if fD and "cofactors" in fD else 0
+        logger.info(
+            "Completed reload of (%d) cofactors with status (%r) at %s (%.4f seconds)", numCofactors, ok, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), time.time() - startTime
+        )
         return fD
 
-    def buildCofactorList(self, sequenceMatchFilePath, crmpObj=None, maxActivity=5):
+    def buildCofactorList(self, sequenceMatchFilePath, crmpObj=None, lnmpObj=None, maxActivity=5):
         """Build target cofactor list for the matching entities in the input sequence match file.
 
         Args:
             sequenceMatchFilePath (str): sequence match output file path
-            crmpObj (obj, optional): instance of ChemRefMappingProviderObj()
-            maxActivity (int, optional): maximum number of prioritized activity records per target
+            crmpObj (obj, optional): instance of ChemRefMappingProviderObj(). Defaults to None.
+            lnmpObj (obj, optional): instance of LigandNeighborMappingProviderObj(). Defaults to None.
+            maxActivity (int, optional): maximum number of prioritized activity records per target. Defaults to 5.
 
         Returns:
             bool: True for success or False otherwise
@@ -105,15 +109,6 @@ class PharosTargetCofactorProvider(StashableBase):
         """
         rDL = []
         mD = self.__mU.doImport(sequenceMatchFilePath, fmt="json")
-        #
-        # --- cofactor list
-        chemblIdList = []
-        for queryId, matchDL in mD.items():
-            tS = queryId.split("|")[2]
-            tL = tS.split(",")
-            chemblIdList.extend(tL)
-        chemblIdList = list(set(chemblIdList))
-        logger.info("Total cofactors for matched targets (%d)", len(chemblIdList))
         # ---
         chaP = PharosTargetActivityProvider(cachePath=self.__cachePath, useCache=True)
         #
@@ -122,36 +117,45 @@ class PharosTargetCofactorProvider(StashableBase):
         assignVersion = chaP.getAssignmentVersion()
         for queryId, matchDL in mD.items():
             # "O43508|uniprotId|7987|proteinId|9606|taxId"
-            unpId = queryId.split("|")[0]
-            queryTaxId = queryId.split("|")[4]
-            pharosId = queryId.split("|")[2]
+            qCmtD = self.__decodeComment(queryId)
+            unpId = qCmtD["uniprotId"]
+            queryTaxId = qCmtD["taxId"] if "taxId" in qCmtD else None
+            pharosId = qCmtD["proteinId"]
             if queryTaxId == "-1":
-                logger.debug("Skipping target %r (%r)", unpId, pharosId)
+                logger.debug("Skipping target with missing taxonomy %r (%r)", unpId, pharosId)
                 continue
             #
             if not chaP.hasTargetActivity(pharosId):
-                logger.debug("Skipping target %r (%r)", unpId, pharosId)
-                continue
+                logger.debug("Skipping target with no activities %r (%r)", unpId, pharosId)
+                # continue
+            # --
+            chemCompNeighborsD = {}
+            if lnmpObj:
+                for matchD in matchDL:
+                    tCmtD = self.__decodeComment(matchD["target"])
+                    entryId = tCmtD["entityId"].split("_")[0]
+                    entityId = tCmtD["entityId"].split("_")[1]
+                    rcsbEntityId = entryId + "_" + entityId
+                    chemCompIdList = lnmpObj.getLigandNeighbors(rcsbEntityId)
+                    chemCompNeighborsD.update({k: True for k in chemCompIdList})
             # --
             queryName = chaP.getTargetInfo(pharosId, "name")
             # --
             for matchD in matchDL:
-                tL = matchD["target"].split("|")
-                entryId = tL[0].split("_")[0]
-                entityId = tL[0].split("_")[1]
+                tCmtD = self.__decodeComment(matchD["target"])
+                entryId = tCmtD["entityId"].split("_")[0]
+                entityId = tCmtD["entityId"].split("_")[1]
+                rcsbEntityId = entryId + "_" + entityId
                 #
                 taDL = chaP.getTargetActivity(pharosId)
                 logger.debug("Target %r has (%d) activity records", pharosId, len(taDL))
                 actL = []
-                cfDL = []
+                # cfDL = []
                 chD = {}
                 for taD in taDL:
                     if taD["chemblId"] in chD:
                         chD[taD["chemblId"]] = True
                         continue
-                    #
-                    cfD = {"cofactor_id": taD["chemblId"]}
-                    cfDL.append(self.__addLocalIds(cfD, crmpObj))
 
                     actD = {
                         "cofactor_id": taD["chemblId"],
@@ -164,9 +168,12 @@ class PharosTargetCofactorProvider(StashableBase):
                         "action": taD["action"] if "action" in taD else None,
                         "pharmacology": taD["pharmacology"] if "pharmacology" in taD else None,
                     }
+                    actD = self.__addLocalIds(actD, crmpObj=crmpObj)
                     actL.append(actD)
                 #
-                actL = self.__activityListSelect(actL, crmpObj, maxActivity=maxActivity)
+                actL = self.__activityListSelect(actL, chemCompNeighborsD, maxActivity=maxActivity)
+                if not actL:
+                    logger.debug("No Pharos cofactors for %s %s", pharosId, unpId)
                 # ---
                 rD = {
                     "entry_id": entryId,
@@ -178,8 +185,8 @@ class PharosTargetCofactorProvider(StashableBase):
                     "provenance_source": provenanceSource,
                     "reference_scheme": refScheme,
                     "assignment_version": assignVersion,
-                    "query_taxonomy_id": int(queryTaxId),
-                    "target_taxonomy_id": int(matchD["targetTaxId"]),
+                    "query_taxonomy_id": int(queryTaxId) if queryTaxId else None,
+                    "target_taxonomy_id": int(matchD["targetTaxId"]) if "targetTaxId" in matchD else None,
                     "target_beg_seq_id": matchD["targetStart"],
                     "query_beg_seq_id": matchD["queryStart"],
                     "align_length": matchD["alignLen"],
@@ -203,10 +210,10 @@ class PharosTargetCofactorProvider(StashableBase):
         ok = self.__mU.doExport(fp, {"version": vS, "created": tS, "cofactors": qD}, fmt="json", indent=3)
         return ok
 
-    def __addLocalIds(self, cfD, crmpOb=None):
+    def __addLocalIds(self, cfD, crmpObj=None):
         #
-        if crmpOb:
-            localIdL = crmpOb.getLocalIds("CHEMBL", cfD["cofactor_id"])
+        if crmpObj:
+            localIdL = crmpObj.getLocalIds("CHEMBL", cfD["cofactor_id"])
             if localIdL:
                 localId = localIdL[0]
                 if localId.startswith("PRD_"):
@@ -215,40 +222,49 @@ class PharosTargetCofactorProvider(StashableBase):
                     cfD["chem_comp_id"] = localId
         return cfD
 
-    def __activityListSelect(self, activityDL, crmpOb=None, maxActivity=5):
-        """Prioritizing the activity data for locally mapped ligands and best binding examples.
+    def __activityListSelect(self, activityDL, chemCompNeighborsD, maxActivity=5):
+        """Prioritizing the activity data for locally mapped neighbor ligands and the best binding examples.
 
         Args:
             activityDL (list): full list of activity objects
-            crmpOb (obj, optional): instance of ChemRefMappingProvider(). Defaults to None.
+            chemCompNeighborsD (dict, optional): index of all chemical components with neighbor interactions to the query target. Defaults {}.
             maxCount (int, optional): maximum number of activity object returned. Defaults to 5.
 
         Returns:
             list: prioritized and trimmed list of activity objects
         """
         retL = []
-        mappedL = []
+        mappedNeighborL = []
         unmappedL = activityDL
-        if crmpOb:
+
+        if chemCompNeighborsD:
             unmappedL = []
-            # Select out the any cases for molecules that map to a local CC or BIRD
+            # Select out the any cases for molecules that map to a neighbor chemical component.
             for activityD in activityDL:
-                localIdL = crmpOb.getLocalIds("CHEMBL", activityD["cofactor_id"])
-                if localIdL:
-                    # add the local identifiers
-                    for localId in localIdL:
-                        if localId.startswith("PRD_"):
-                            activityD.setdefault("prd_id", []).append(localId)
-                        else:
-                            activityD.setdefault("chem_comp_id", []).append(localId)
-                    mappedL.append(activityD)
+                if "chem_comp_id" in activityD and activityD["chem_comp_id"] in chemCompNeighborsD:
+                    activityD["neighbor_in_pdb"] = "Y"
+                    mappedNeighborL.append(activityD)
                 else:
                     unmappedL.append(activityD)
+                    activityD["neighbor_in_pdb"] = "N"
         #
-        numLeft = maxActivity - len(mappedL)
+        numLeft = maxActivity - len(mappedNeighborL)
         if numLeft > 0:
             unmappedL = sorted(unmappedL, key=lambda k: k["measurement_value"], reverse=True)
-            retL = mappedL
+            retL = mappedNeighborL
             retL.extend(unmappedL[:numLeft])
             retL = sorted(retL, key=lambda k: k["measurement_value"], reverse=True)
+        else:
+            logger.debug("Mapped neighbor cofactors (%d) excluded unmapped (%d)", len(mappedNeighborL), len(unmappedL))
+            retL = sorted(mappedNeighborL, key=lambda k: k["measurement_value"], reverse=True)
+
         return retL
+
+    def __decodeComment(self, comment, separator="|"):
+        dD = {}
+        try:
+            ti = iter(comment.split(separator))
+            dD = {tup[1]: tup[0] for tup in zip(ti, ti)}
+        except Exception:
+            pass
+        return dD
